@@ -20,14 +20,12 @@ from .sloth import Sloth
 def make_listener(sloth):
 
     @cherrypy.expose
-    def listener(payload, orig=True):
-        """Listens to payloads.
+    def listener(*args, **kwargs):
+        """Listens to requests.
 
         :param payload: payload
         """
 
-        if not cherrypy.request.method == 'POST':
-            raise cherrypy.HTTPError(405)
         sloth.logger.info('Payload received from %s - %s' % (cherrypy.request.headers['Remote-Addr'], cherrypy.request.headers['User-Agent']))
 
         try:
@@ -35,21 +33,19 @@ def make_listener(sloth):
                 '.validators.%s' % sloth.config['provider'],
                 package=__package__
             )
-
         except ImportError as e:
             sloth.logger.critical('No matching validator found: %s' % e)
             raise cherrypy.HTTPError(500)
 
-        payload_valid, validation_message = validator.validate(payload, sloth.config['provider_data'])
+        payload_valid, validation_message, params = validator.validate(cherrypy.request, sloth.config['provider_data'])
 
-        sloth.logger.info(validation_message)
+        sloth.logger.info(validation_message.format_map(params))
 
         if not payload_valid:
             raise cherrypy.HTTPError(400)
 
-
         if not sloth.is_queue_locked():
-            sloth.queue.append((payload, orig))
+            sloth.queue.append(params)
 
     return listener
 
@@ -62,7 +58,6 @@ def run(host, port, log_dir, sloths):
 
     cherrypy.config.update(
         {
-            'environment': 'production',
             'server.socket_host': host,
             'server.socket_port': port,
             'log.access_file': abspath(join(log_dir, 'access.log')),
@@ -87,15 +82,26 @@ def main():
     """Main API function"""
 
     parser = ArgumentParser()
-    parser.add_argument('--host', required=True)
-    parser.add_argument('--port', type=int, required=True)
-    parser.add_argument('--log_dir', required=True)
+    parser.add_argument('--sconfig')
+    parser.add_argument('--host')
+    parser.add_argument('--port', type=int)
+    parser.add_argument('--log_dir')
     parser.add_argument('config', nargs='+')
 
-    host, port, log_dir = parser.parse_args().host, parser.parse_args().port, parser.parse_args().log_dir
+    parsed_args = parser.parse_args()
 
-    config_files = parser.parse_args().config
+    sconfig_file = parsed_args.sconfig
+    sconfig = load(sconfig_file)
+
+    host = parsed_args.host or sconfig.get('host')
+    port = parsed_args.port or sconfig.get('port')
+    log_dir = parsed_args.log_dir or sconfig.get('log_dir')
+
+    config_files = parsed_args.config
 
     sloths = [Sloth(load(config_file, defaults={'log_dir': log_dir})) for config_file in config_files]
 
+    if not (host or port or log_dir):
+        raise RuntimeError('Missing server param.')
+
     run(host, port, log_dir, sloths)
